using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using CommerceBuilder.Common;
using CommerceBuilder.Orders;
using CommerceBuilder.Utility;

public partial class Admin_Orders_OrderHistory : CommerceBuilder.Web.UI.AbleCommerceAdminPage
{
    private int _OrderId = 0;
    private Order _Order;

    protected void Page_Load(object sender, EventArgs e)
    {
        _Order = OrderHelper.GetOrderFromContext();
        if (_Order != null)
        {
            _OrderId = AlwaysConvert.ToInt(Request.QueryString["OrderId"]);
            if (!Page.IsPostBack)
            {
                BindOrderNotes();
                AddIsPrivate.Checked = true;
            }
        }
        AddDialogCaption.Text = string.Format(AddDialogCaption.Text, _Order.OrderNumber);
        EditDialogCaption.Text = string.Format(EditDialogCaption.Text, _Order.OrderNumber);
    }

    protected string GetAuthor(object dataItem)
    {
        OrderNote note = (OrderNote)dataItem;
        if (note.NoteType == NoteType.SystemPrivate || note.NoteType == NoteType.SystemPublic) return "system";
        return note.User.Email;
    }

    protected void BindOrderNotes()
    {
        OrderNoteCollection notes = _Order.Notes;
        notes.Sort("CreatedDate DESC");
        OrderNotesGrid.DataSource = notes;
        OrderNotesGrid.DataBind();
    }

    protected void ShowAddDialog_Click(object sender, EventArgs e)
    {
        AddPopup.Show();
        AddComment.Focus();
    }

    protected void MarkAllReadButton_Click(object sender, EventArgs e)
    {
        foreach (OrderNote note in _Order.Notes)
        {
            note.IsRead = true;
        }
        _Order.Save();
        BindOrderNotes();
    }

    protected void AddButton_Click(object sender, EventArgs e)
    {
        if (!string.IsNullOrEmpty(AddComment.Text))
        {
            _Order.Notes.Add(new OrderNote(_OrderId, Token.Instance.UserId, LocaleHelper.LocalNow, AddComment.Text, AddIsPrivate.Checked ? NoteType.Private : NoteType.Public));
            

            if (MarkAllRead.Checked)
            {
                foreach (OrderNote note in _Order.Notes)
                {
                    note.IsRead = true;
                }
            }
            _Order.Save();

            BindOrderNotes();
            AddComment.Text = string.Empty;
            MarkAllRead.Checked = true;
        }
        AddIsPrivate.Checked = true;
    }

    protected void EditButton_Click(object sender, EventArgs e)
    {
        int orderNoteId = AlwaysConvert.ToInt(EditId.Value);
        OrderNote n = OrderNoteDataSource.Load(orderNoteId);
        if (n != null && !string.IsNullOrEmpty(EditComment.Text))
        {
            n.Comment = EditComment.Text;
            n.NoteType = EditIsPrivate.Checked ? NoteType.Private : NoteType.Public;
            n.IsRead = EditIsRead.Checked;
            n.Save();
            BindOrderNotes();
        }
    }

    protected void OrderNotesGrid_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        int orderNoteId = AlwaysConvert.ToInt(e.CommandArgument);
        if (e.CommandName == "EditNote")
        {
            OrderNote n = OrderNoteDataSource.Load(orderNoteId);
            EditComment.Text = n.Comment;
            EditIsPrivate.Checked = n.IsPrivate;
            EditIsRead.Checked = n.IsRead;
            EditId.Value = orderNoteId.ToString();
            EditPopup.Show();
        }
        else if (e.CommandName == "DeleteNote")
        {
            int index = _Order.Notes.IndexOf(orderNoteId);
            if (index > -1) _Order.Notes.DeleteAt(index);
            BindOrderNotes();
        }
    }
    
    protected void Page_PreRender(object sender, EventArgs e)
    {
        int unreadCount = 0;
        foreach (OrderNote note in _Order.Notes)
        {
            if (!note.IsRead) unreadCount++;
        }
        MarkAllReadButton.Visible = (unreadCount > 0);
    }
}
