using System;
using System.Collections;
using System.Collections.Generic;
using System.Net.Mail;
using System.Web.UI;
using System.Web.UI.WebControls;
using CommerceBuilder.Common;
using CommerceBuilder.Messaging;
using CommerceBuilder.Orders;
using CommerceBuilder.Stores;
using CommerceBuilder.Users;
using CommerceBuilder.Utility;

public partial class Admin_Reports_DailyAbandonedBaskets : CommerceBuilder.Web.UI.AbleCommerceAdminPage
{
    StoreSettingCollection _Settings;
    protected void Page_Load(object sender, EventArgs e)
    {
        _Settings = Store.GetCachedSettings();
        if (_Settings.AbandonedBasketAlertEmailTemplateId == 0)
        {
            NotePlaceHolder.Controls.Add(new LiteralControl("<p style='text-align:left;'>We noticed that you don't have any Email template configured for abandoned basket alerts. Please choose suitable template in <b>Administration > Configure > Email > Settings</b> section.</p>"));
        }

        if (!Page.IsPostBack)
        {
            
            DateTime tempDate = AlwaysConvert.ToDateTime(Request.QueryString["Date"], System.DateTime.MinValue);
            if (tempDate == System.DateTime.MinValue) tempDate = LocaleHelper.LocalNow;
            ViewState["ReportDate"] = tempDate;
            ReportDate.SelectedDate = tempDate;
            BindReport();
        }
    }

    protected void BindReport() 
    {
        DateTime reportDate = AlwaysConvert.ToDateTime(ViewState["ReportDate"],DateTime.MinValue);
        List<CommerceBuilder.Reporting.BasketSummary> abandonedBaskets = CommerceBuilder.Reporting.ReportDataSource.GetDailyAbandonedBaskets(reportDate.Year, reportDate.Month, reportDate.Day);
        DailyAbandonedBasketsGrid.DataSource = abandonedBaskets;
        DailyAbandonedBasketsGrid.DataBind();
        ReportCaption.Visible = true;
        ReportCaption.Text = string.Format(ReportCaption.Text, ReportDate.SelectedDate);
    }
    
    protected int GetUserId(int basketId)
    {
        Basket basket = BasketDataSource.Load(basketId);
        if (basket != null)
        {
            return basket.UserId;
        }
        return basketId;
    }

    protected void ProcessButton_Click(Object sender, EventArgs e)
    {
        DateTime newReportDate = ReportDate.SelectedDate;
        ViewState["ReportDate"] = newReportDate;
        BindReport();
    }

    protected bool HasEmail(Object value)
    {
        int basketId = AlwaysConvert.ToInt(value);
        Basket basket = BasketDataSource.Load(basketId);
        if (basket.User.IsAnonymous)
        {
            if (basket.User.PrimaryAddress != null && !string.IsNullOrEmpty(basket.User.PrimaryAddress.Email))
                return true;
            return false;
        }
        return true;
    }

    protected void DailyAbandonedBasketsGrid_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        if (e.CommandName == "SENDALERT")
        {
            int basketId = AlwaysConvert.ToInt(e.CommandArgument);
            Basket basket = BasketDataSource.Load(basketId);
            if (basket != null)
            {
                int templateId = _Settings.AbandonedBasketAlertEmailTemplateId;
                EmailTemplate emailTemplate = EmailTemplateDataSource.Load(templateId);
                if (emailTemplate != null)
                {
                    Hashtable parameters = new Hashtable();
                    parameters.Add("store", Token.Instance.Store);
                    parameters.Add("customer", basket.User);
                    parameters.Add("basket", basket);

                    string messageContents = NVelocityEngine.Instance.Process(parameters, emailTemplate.Body);
                    string toEmail = basket.User.Email;
                    if (!ValidationHelper.IsValidEmail(toEmail))
                        toEmail = basket.User.UserName;
                    if (!ValidationHelper.IsValidEmail(toEmail))
                        toEmail = basket.User.PrimaryAddress.Email;

                    ToEmail.Text = toEmail;
                    Subject.Text = NVelocityEngine.Instance.Process(parameters, emailTemplate.Subject);
                    MessageContents.Value = messageContents;
                    SendPopup.Show();
                }
            }
                        
        }
    }

    protected string GetCustomerName(object dataItem)
    {
        CommerceBuilder.Reporting.BasketSummary summary = (CommerceBuilder.Reporting.BasketSummary)dataItem;
        Basket basket = BasketDataSource.Load(summary.BasketId);
        User user = basket.User;
        string fullName = user.PrimaryAddress.FullName;
        if (!string.IsNullOrEmpty(fullName)) return fullName;
        if (!string.IsNullOrEmpty(user.Email)) return user.Email;
        if (AlwaysConvert.ToGuid(user.UserName) == Guid.Empty) return "Anonymous";
        return user.UserName;
    }

    protected void SendButton_Click(Object sender,EventArgs e) 
    {
        ReportAjax.Update();
        try
        {
            EmailTemplate emailTemplate = EmailTemplateDataSource.Load(_Settings.AbandonedBasketAlertEmailTemplateId);
            MailMessage mailMessage = new MailMessage();
            mailMessage.To.Add(new MailAddress(ToEmail.Text));
            mailMessage.From = new MailAddress(emailTemplate.FromAddress);

            string[] ccAddresses = emailTemplate.CCList.Split(',');
            if (ccAddresses != null && ccAddresses.Length > 0)
            {
                foreach (string ccAddress in ccAddresses)
                {
                    if(ValidationHelper.IsValidEmail(ccAddress))
                        mailMessage.CC.Add(new MailAddress(ccAddress.Trim()));
                }
            }

            string[] bccAddresses = emailTemplate.BCCList.Split(',');
            if (bccAddresses != null && bccAddresses.Length > 0)
            {
                foreach (string bccAddress in bccAddresses)
                {
                    if (ValidationHelper.IsValidEmail(bccAddress))
                        mailMessage.Bcc.Add(new MailAddress(bccAddress.Trim()));
                }
            }

            mailMessage.Subject = Subject.Text;
            mailMessage.Body = MessageContents.Value;
            mailMessage.IsBodyHtml = emailTemplate.IsHTML;
            EmailClient.Send(mailMessage);
        }
        catch (Exception exp)
        {
            EmailFailedMessage.Text = "Unable to send Email, make sure your SMTP settings are configured properly. Please view applicatoin error log for more details.";
            Logger.Error(exp.Message);
            SendPopup.Show();
        }
    }
}
