using CommerceBuilder.Shipping.Providers.FedExWS;
using System;
using CommerceBuilder.Shipping;
using CommerceBuilder.Utility;
using System.Web;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using CommerceBuilder.Shipping.Providers.FedExWS.RateService9;

partial class Admin_Shipping_Providers_FedExWS_Configure : CommerceBuilder.Web.UI.AbleCommerceAdminPage
{

    private int _ShipGatewayId = 0;
    private ShipGateway _ShipGateway;

    int ShipGatewayId
    {
        get
        {
            if (_ShipGatewayId.Equals(0))
            {
                _ShipGatewayId = AlwaysConvert.ToInt(Request.QueryString["ShipGatewayId"]);
            }
            return _ShipGatewayId;
        }
    }

    ShipGateway ShipGateway
    {
        get
        {
            if ((_ShipGateway == null))
            {
                _ShipGateway = new ShipGateway();
                if (!_ShipGateway.Load(ShipGatewayId))
                {
                    _ShipGateway = null;
                }
            }
            return _ShipGateway;
        }
    }

    protected string ClassId
    {
        get
        {
            return Misc.GetClassId(typeof(FedExWS));
        }
    }

    protected void Page_Load(object sender, System.EventArgs e)
    {
        // MAKE SURE A VALID SHIPGATEWAY ID IS FOUND
        if ((ShipGateway == null))
        {
            RedirectToManageProvider();
        }
        // GET THE PROVIDER INSTANCE AND MAKE SURE IT IS ACTIVE
        FedExWS provider = (FedExWS)ShipGateway.GetProviderInstance();
        if ((provider == null))
        {
            // THIS GATEWAY DOES NOT HAVE A FedExWS PROVIDER INSTANCE (UNEXPECTED)
            // Response.Redirect("Default.aspx")
            RedirectToManageProvider();
        }

        if (!Page.IsPostBack)
        {
            // INITIALIZE THE FORM FIELDS
            UseDebugMode.SelectedValue = (provider.UseDebugMode ? "1" : "0");
            UseTestMode.SelectedValue = (provider.UseTestMode ? "1" : "0");
            EnablePackageBreakup.SelectedValue = (provider.EnablePackageBreakup ? "1" : "0");
            IncludeDeclaredValue.SelectedValue = (provider.IncludeDeclaredValue ? "1" : "0");
            FedExKey.Text = provider.Key;
            FedExPassword.Text = provider.Password;
            AccountNameDisplay.Text = provider.AccountNumber;
            MeterNumberDisplay.Text = provider.MeterNumber;
            InstanceName.Text = _ShipGateway.Name;
            BindDropOffType(provider.DropoffType);
            BindPackagingType(provider.PackagingType);
            MaxPackageWeight.Text = provider.MaxPackageWeight.ToString();
            MinPackageWeight.Text = provider.MinPackageWeight.ToString();
            TrackingURL.Text = provider.TrackingUrl;

            //DropOffType.SelectedIndex

            //CHECK WHETHER ANY PROVIDERS HAVE BEEN CONFIGURED
            ShipGatewayCollection configuredProviders = ShipGatewayDataSource.LoadForClassId(ShipGateway.ClassId);
            if (configuredProviders.Count > 1)
            {
                trInstanceName.Visible = true;
            }
        }

        IntegrationVersion.Text = provider.Version;
    }

    protected void Page_PreRender(object sender, System.EventArgs e)
    {
        TestModeMessage.Visible = UseTestMode.SelectedValue == "1";
    }

    protected void RedirectToManageProvider()
    {
        FedExWS provider;
        provider = new FedExWS();
        Response.Redirect(("../Default.aspx?ProviderClassID=" + HttpUtility.UrlEncode(Misc.GetClassId(provider.GetType()))));
    }

    protected void CancelButton_Click(object sender, System.EventArgs e)
    {
        RedirectToManageProvider();
    }

    protected void SaveButton_Click(object sender, System.EventArgs e)
    {
        FedExWS provider = (FedExWS)ShipGateway.GetProviderInstance();
        provider.UseDebugMode = (UseDebugMode.SelectedValue == "1");
        provider.UseTestMode = (UseTestMode.SelectedValue == "1");
        provider.EnablePackageBreakup = (EnablePackageBreakup.SelectedValue == "1");
        provider.IncludeDeclaredValue = (IncludeDeclaredValue.SelectedValue == "1");
        provider.Key = FedExKey.Text;
        provider.Password = FedExPassword.Text;
        provider.AccountNumber = AccountNameDisplay.Text;
        provider.MeterNumber = MeterNumberDisplay.Text;
        provider.DropoffType = (DropoffType)AlwaysConvert.ToEnum(typeof(DropoffType), DropOffType.SelectedValue, DropoffType.REGULAR_PICKUP);
        provider.PackagingType = (PackagingType)AlwaysConvert.ToEnum(typeof(PackagingType), PackagingType.SelectedValue, CommerceBuilder.Shipping.Providers.FedExWS.RateService9.PackagingType.YOUR_PACKAGING);
        provider.MaxPackageWeight = AlwaysConvert.ToDecimal(MaxPackageWeight.Text, (decimal)provider.MaxPackageWeight);
        provider.MinPackageWeight = AlwaysConvert.ToDecimal(MinPackageWeight.Text, (decimal)provider.MinPackageWeight);
        provider.TrackingUrl = TrackingURL.Text;
        ShipGateway.UpdateConfigData(provider.GetConfigData());
        ShipGateway.Name = InstanceName.Text;
        ShipGateway.Save();
    }

    protected void DeleteButton_Click(object sender, System.EventArgs e)
    {
        ShipGateway.Delete();
        Response.Redirect("DeleteConfirm.aspx");
    }

    protected void BindDropOffType(DropoffType dType)
    {
        DropOffType.Items.Clear();
        foreach (DropoffType dOffType in Enum.GetValues(typeof(DropoffType)))
        {
            ListItem newItem = new ListItem(dOffType.ToString(), dOffType.ToString());
            DropOffType.Items.Add(newItem);
            if (dOffType == dType)
            {
                newItem.Selected = true;
            }
        }
    }

    protected void BindPackagingType(PackagingType pType)
    {
        PackagingType.Items.Clear();
        foreach (PackagingType pkgType in Enum.GetValues(typeof(PackagingType)))
        {
            ListItem newItem = new ListItem(pkgType.ToString(), pkgType.ToString());
            PackagingType.Items.Add(newItem);
            if (pkgType == pType)
            {
                newItem.Selected = true;
            }
        }
    }
}
