using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Web.UI;
using System.Web.UI.WebControls;
using CommerceBuilder.Common;
using CommerceBuilder.Data;
using CommerceBuilder.Search;
using CommerceBuilder.Stores;
using CommerceBuilder.Users;
using CommerceBuilder.Utility;

public partial class Admin_Store_Maintenance : CommerceBuilder.Web.UI.AbleCommerceAdminPage
{
    protected void Page_Init(object sender, EventArgs e)
    {
        // INCREASE TIMEOUT FOR PAGE PROCESSING
        Server.ScriptTimeout = 7200;
    }

    protected void Page_Load(object sender, System.EventArgs e)
    {
        if (!Page.IsPostBack)
        {
            Store store = Token.Instance.Store;
            StoreSettingCollection settings = store.Settings;
                    
            StoreClosedOptions.DataSource = EnumToDictionary(typeof(StoreClosureType));
            StoreClosedOptions.DataTextField = "Key";
            StoreClosedOptions.DataValueField = "Value";
            StoreClosedOptions.DataBind();
            ListItem option = StoreClosedOptions.Items.FindByValue(((int)settings.StoreFrontClosed).ToString());
            if (option != null) option.Selected = true;
            
            StoreClosedMessage.Text = settings.StoreFrontClosedMessage;
                       
            //USER MAINTENANCE
            if (settings.AnonymousUserLifespan > 0) AnonymousLifespan.Text = settings.AnonymousUserLifespan.ToString();
            if (settings.AnonymousAffiliateUserLifespan > 0) AnonymousAffiliateLifespan.Text = settings.AnonymousAffiliateUserLifespan.ToString();

			//GIFT CERTIFICATE EXPIRY
            GiftCertExpireDays.Text = settings.GiftCertificateDaysToExpire.ToString();
			
            // SHOW REBUILD INDEX ROW IF FTS IS ENABLED FOR STORE
            trRebuildIndex.Visible = Store.GetCachedSettings().FullTextSearch;
        }
    }

    private void InitAnonUserStatus()
    {
        // GET COUNT OF ANONYMOUS USERS
        int anonUserCount = UserDataSource.CountForCriteria("IsAnonymous = 1 AND AffiliateId IS NULL");
        AnonymousUserCount.Text = anonUserCount.ToString();
        UserCollection anonUsers = UserDataSource.LoadForCriteria("IsAnonymous = 1 AND AffiliateId IS NULL AND LastActivityDate IS NOT NULL", 1, 0, "LastActivityDate");
        if (anonUsers != null && anonUsers.Count > 0)
        {
            OldestAnonUser.Text = GetElapsedDays(anonUsers[0].LastActivityDate).ToString() + " days @ " + anonUsers[0].LastActivityDate.ToString();
        }
        else OldestAnonUser.Text = "n/a";
        int anonAffiliateUserCount = UserDataSource.CountForCriteria("IsAnonymous = 1 AND AffiliateId IS NOT NULL");
        AffiliateAnonymousUserCount.Text = anonAffiliateUserCount.ToString();
        anonUsers = UserDataSource.LoadForCriteria("IsAnonymous = 1 AND AffiliateId IS NOT NULL AND LastActivityDate IS NOT NULL", 1, 0, "LastActivityDate");
        if (anonUsers != null && anonUsers.Count > 0)
        {
            OldestAffAnonUser.Text = GetElapsedDays(anonUsers[0].LastActivityDate).ToString() + " days @ " + anonUsers[0].LastActivityDate.ToString();
        }
        else OldestAffAnonUser.Text = "n/a";
    }

    private int GetElapsedDays(DateTime pastDate)
    {
        TimeSpan ts = DateTime.UtcNow - pastDate.ToUniversalTime();
        return ts.Days;
    }

    protected Dictionary<string,int> EnumToDictionary(Type enumType)
    {
        // get the names from the enumeration
        string[] names = Enum.GetNames(enumType);
        // get the values from the enumeration
        Array values = Enum.GetValues(enumType);
        // turn it into a dictionary
        Dictionary<string,int> ht = new Dictionary<string,int>();
        for (int i = 0; i < names.Length; i++)
            // note the cast to integer here is important
            // otherwise we'll just get the enum string back again
			ht.Add(StringHelper.SpaceName(names[i]), (int)values.GetValue(i));
        // return the dictionary to be bound to
        return ht;
    }

    protected void CancelButton_Click(object sender, EventArgs e)
    {
        Response.Redirect("../Default.aspx");
    }

    private void SaveSettings()
    {
        if (Page.IsValid)
        {
            Store store = Token.Instance.Store;
            StoreSettingCollection settings = store.Settings;

            settings.StoreFrontClosed = (StoreClosureType)AlwaysConvert.ToInt(StoreClosedOptions.SelectedValue);
            settings.StoreFrontClosedMessage = StoreClosedMessage.Text;

            //USER MAINTENANCE
            settings.AnonymousUserLifespan = AlwaysConvert.ToInt(AnonymousLifespan.Text);
            settings.AnonymousAffiliateUserLifespan = AlwaysConvert.ToInt(AnonymousAffiliateLifespan.Text);
            settings.GiftCertificateDaysToExpire = AlwaysConvert.ToInt(GiftCertExpireDays.Text);
            store.Save();
        }
    }
    
    protected void SaveButton_Click(object sender, EventArgs e)
    {
        SaveSettings();
        SavedMessage.Visible = true;
    }
    
    protected void SaveAndCloseButton_Click(object sender, EventArgs e) {
        SaveSettings();
        Response.Redirect("../Default.aspx");
    }

    protected void RebuildIndexButton_Click(object sender, EventArgs e)
    {
        KeywordSearchHelper.RebuildCatalog();
        RebuildIndexMessage.Visible = true;
    }

    protected void CheckDatabaseButton_Click(object sender, EventArgs e)
    {
        // PRINT ANON USER STATUS
        InitAnonUserStatus();
        InstructionsPanel.Visible = false;
        AnonymousUsersDataPanel.Visible = true;
    }

    protected void ManualCleanupButton_Click(object sender, EventArgs e)
    {
        if (Page.IsValid)
        {
            Database database = Token.Instance.Database;
            int anonLifespan = AlwaysConvert.ToInt(ManualCleanupDays1.Text);
            if (anonLifespan > 0 && anonLifespan < 10000)
            {
                DateTime expireDate = DateTime.UtcNow.AddDays(-1 * anonLifespan);
                expireDate = new DateTime(expireDate.Year, expireDate.Month, expireDate.Day, 0, 0, 0);
                string sql = @"CREATE TABLE #ExpiredAnonUsers (UserId INT PRIMARY KEY);
INSERT INTO #ExpiredAnonUsers 
SELECT UserId FROM ac_Users
WHERE StoreId = @storeId
AND IsAnonymous = 1 
AND AffiliateId IS NULL
AND LastActivityDate < @lastActivityDate;
DELETE ac_Baskets
FROM ac_Baskets AS T1
INNER JOIN #ExpiredAnonUsers AS T2
ON T1.UserId = T2.UserId;
DELETE ac_Wishlists
FROM ac_Wishlists AS T1
INNER JOIN #ExpiredAnonUsers AS T2
ON T1.UserId = T2.UserId;
DELETE ac_Users
FROM ac_Users AS T1
INNER JOIN #ExpiredAnonUsers AS T2
ON T1.UserId = T2.UserId;
DROP TABLE #ExpiredAnonUsers;";
                DbCommand command = database.GetSqlStringCommand(sql);
                database.AddInParameter(command, "storeId", DbType.Int32, Token.Instance.StoreId);
                database.AddInParameter(command, "lastActivityDate", DbType.DateTime, expireDate);
                database.ExecuteNonQuery(command);
            }

            anonLifespan = AlwaysConvert.ToInt(ManualCleanupDays2.Text);
            if (anonLifespan > 0 && anonLifespan < 10000)
            {
                DateTime expireDate = DateTime.UtcNow.AddDays(-1 * anonLifespan);
                expireDate = new DateTime(expireDate.Year, expireDate.Month, expireDate.Day, 0, 0, 0);
                string sql = @"CREATE TABLE #ExpiredAnonUsers (UserId INT PRIMARY KEY);
INSERT INTO #ExpiredAnonUsers 
SELECT UserId FROM ac_Users
WHERE StoreId = @storeId
AND IsAnonymous = 1 
AND AffiliateId IS NOT NULL
AND LastActivityDate < @lastActivityDate;
DELETE ac_Baskets
FROM ac_Baskets AS T1
INNER JOIN #ExpiredAnonUsers AS T2
ON T1.UserId = T2.UserId;
DELETE ac_Wishlists
FROM ac_Wishlists AS T1
INNER JOIN #ExpiredAnonUsers AS T2
ON T1.UserId = T2.UserId;
DELETE ac_Users
FROM ac_Users AS T1
INNER JOIN #ExpiredAnonUsers AS T2
ON T1.UserId = T2.UserId;
DROP TABLE #ExpiredAnonUsers;";
                DbCommand command = database.GetSqlStringCommand(sql);
                database.AddInParameter(command, "storeId", DbType.Int32, Token.Instance.StoreId);
                database.AddInParameter(command, "lastActivityDate", DbType.DateTime, expireDate);
                database.ExecuteNonQuery(command);
            }

            if (AnonymousUsersDataPanel.Visible)
            {
                // UPDATE ANON USER STATUS
                InitAnonUserStatus();
            }
        }
    }

    protected void Page_PreRender(object sender, EventArgs e)
    {
        StoreSettingCollection settings = Token.Instance.Store.Settings;
        phAnonymousLifespanWarning.Visible = (settings.AnonymousUserLifespan == 0 && settings.AnonymousAffiliateUserLifespan == 0);
    }
}
